﻿using System;
using System.Data;
using System.Configuration;
using System.Linq;
using System.Web;
using System.Web.UI;
using System.Web.UI.HtmlControls;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;

using Telerik.Web.UI;
using System.Collections.Generic;

using System.Reflection;
using System.Collections;

namespace Telerik.Web.UI.CustomExtensions
{
    public class SuggestiveFilteringColumn : GridBoundColumn
    {
        #region Properties

        /// <summary>
        /// The filter function that will be used to compare
        /// the selected filter
        /// </summary>
        public GridKnownFunction FilterFunction
        {
            get
            {
                if (ViewState["FF"] == null)
                    return GridKnownFunction.EqualTo;
                else
                    return (GridKnownFunction)ViewState["FF"];
            }
            set{ ViewState["FF"] = value; }
        }
        private List<string> DistinctColumnValues
        {
            get
            {
                string key = "DCV";
                if (ViewState[key] == null)
                    ViewState[key] = new List<string>();

                return (List<string>)ViewState[key];
            }
        }
        private string RadComboFilterID
        {
            get { return "RadComboBox1" + this.DataField; }
        }

        #endregion

        #region Owner Events

        private void Owner_DataBound(object sender, EventArgs e)
        {
            GridTableView gtv = (GridTableView)sender;

            if( gtv.DataSource != null )
                CacheDistinctColumnValues(gtv.DataSource);
        }

        #endregion

        #region Overrides

        protected override void SetupFilterControls(TableCell cell)
        {
            base.SetupFilterControls(cell);

            cell.Controls.RemoveAt(0);
            RadComboBox combo = TheFilterComboBox();
            cell.Controls.AddAt(0, combo);
            cell.Controls.RemoveAt(1);
            cell.Controls.Add(new LiteralControl(@"<span style=""margin-left:10px;"">"));
            cell.Controls.Add(this.ClearFilterButton());
            cell.Controls.Add(new LiteralControl(@"</span>"));

            this.Owner.DataBound += new EventHandler(Owner_DataBound);
        }

        protected override void SetCurrentFilterValueToControl(TableCell cell)
        {
            base.SetCurrentFilterValueToControl(cell);
            RadComboBox combo = (RadComboBox)cell.Controls[0];

            if (this.CurrentFilterValue != string.Empty)
                combo.Text = this.CurrentFilterValue;
        }
        protected override string GetCurrentFilterValueFromControl(TableCell cell)
        {
            RadComboBox combo = (RadComboBox)cell.Controls[0];
            return combo.Text;
        }

        #endregion

        #region Dynamic Control Events

        private void list_ItemsRequested(object o, RadComboBoxItemsRequestedEventArgs e)
        {
            RadComboBox r = (RadComboBox)o;
            r.DataSource = DistinctColumnValues;
            r.DataBind();
        }
        private void list_SelectedIndexChanged(object o, RadComboBoxSelectedIndexChangedEventArgs e)
        {
            GridFilteringItem filterItem = (GridFilteringItem)(o as RadComboBox).NamingContainer;
            FireFilterCommand(filterItem);
        }
        private void clearFilterBtn_Click(object sender, EventArgs e)
        {
            LinkButton btn = (LinkButton)sender;

            this.CurrentFilterValue = String.Empty;
            this.CurrentFilterFunction = GridKnownFunction.NoFilter;

            RadComboBox r = (RadComboBox)btn.Parent.FindControl(this.RadComboFilterID);
            r.Text = "";

            GridFilteringItem filterItem = (GridFilteringItem)btn.NamingContainer;
            FireFilterCommand(filterItem);
        }

        #endregion

        #region Private

        private void FireFilterCommand(GridFilteringItem filterItem)
        {
            //if (this.UniqueName == "Index")
            //    filterItem.FireCommandEvent("Filter", new Pair("EqualTo", this.UniqueName));
            filterItem.FireCommandEvent("Filter", new Pair(FilterFunction.ToString(), this.UniqueName));
        }
        private RadComboBox TheFilterComboBox()
        {
            RadComboBox combo = new RadComboBox();
            combo.ID = this.RadComboFilterID;
            combo.ShowToggleImage = false;
            combo.Skin = "Office2007";
            combo.EnableLoadOnDemand = true;
            combo.AutoPostBack = true;
            combo.MarkFirstMatch = true;
            combo.Height = Unit.Pixel(100);
            combo.ItemsRequested += new RadComboBoxItemsRequestedEventHandler(this.list_ItemsRequested);
            combo.SelectedIndexChanged += new RadComboBoxSelectedIndexChangedEventHandler(this.list_SelectedIndexChanged);

            return combo;
        }
        private LinkButton ClearFilterButton()
        {
            LinkButton clearFilterBtn = new LinkButton();
            clearFilterBtn.Text = "Clear";
            clearFilterBtn.ToolTip = String.Format("Clear the {0} filter", this.UniqueName);
            clearFilterBtn.Click += new EventHandler(clearFilterBtn_Click);
            clearFilterBtn.Visible = (this.CurrentFilterValue.Trim() != "");

            return clearFilterBtn;
        }

        #region Caching of the distinct values

        private void CacheDistinctColumnValues(object source)
        {
            if (source != null)
            {
                Type sourceType = source.GetType();
                if (sourceType.GetInterface(typeof(IEnumerable).Name) != null)
                {
                    foreach (object o in (IEnumerable)source)
                    {
                        AddReflectedDistinctObjectValue(o);
                    }
                }
                else if( sourceType == typeof(DataTable))
                {
                    foreach (DataRow dr in ((DataTable)source).Rows)
                    {
                        object val = dr[this.DataField];
                        if( val != null)
                            this.CacheDistinctColumnValue(val.ToString());
                    }
                }
            }
        }
        private void AddReflectedDistinctObjectValue(object o)
        {
            if (o != null)
            {
                PropertyInfo pi = o.GetType().GetProperty(this.DataField);
                if (pi != null)
                {
                    object val = pi.GetValue(o, null);
                    if (val != null)
                        this.CacheDistinctColumnValue(val.ToString());
                }
            }
        }
        private void CacheDistinctColumnValue(string Text)
        {
            if (Text != null && Text != "&nbsp;" && Text != "")
            {
                if (!this.DistinctColumnValues.Contains(Text))
                    this.DistinctColumnValues.Add(Text);
            }
        }

        #endregion

        #endregion
    }
}